from __future__ import absolute_import


from .modifiers import *  # noqa: F401,F403
from .base import *  # noqa: F401,F403
from .shapes_3d import *  # noqa: F401,F403
from .shapes_2d import *  # noqa: F401,F403
from .boolean import *  # noqa: F401,F403
from .transformations import *  # noqa: F401,F403
from .custom2dshapes import Custom2dShapes  # noqa: F401,F403

__name__ = 'OpenPySCAD'
__version__ = '0.3.2'
__author__ = 'Takuro Wada'
__email__ = 'taxpon@gmail.com'
__url__ = 'https://github.com/taxpon/openpyscad'
__license__ = 'MIT'



# -*- coding: utf-8 -*-


class Modifier(object):

    def __init__(self):
        self.is_disable = False
        self.is_show_only = False
        self.is_debug = False
        self.is_transparent = False

    def turn_on_disable(self):
        self.is_disable = True

    def turn_off_disable(self):
        self.is_disable = False

    def turn_on_show_only(self):
        self.is_show_only = True

    def turn_off_show_only(self):
        self.is_show_only = False

    def turn_on_debug(self):
        self.is_debug = True

    def turn_off_debug(self):
        self.is_debug = False

    def turn_on_transparent(self):
        self.is_transparent = True

    def turn_off_transparent(self):
        self.is_transparent = False

    def get_prefix(self):
        prefix = ''
        if self.is_disable:
            prefix += '*'
        if self.is_show_only:
            prefix += '!'
        if self.is_debug:
            prefix += '#'
        if self.is_transparent:
            prefix += '%'
        return prefix


class ModifierMixin(object):

    def __init__(self):
        super(ModifierMixin, self).__init__()
        self.mod = Modifier()

    def turn_on_disable(self):
        self.mod.is_disable = True
        return self

    def turn_off_disable(self):
        self.mod.is_disable = False
        return self

    def turn_on_show_only(self):
        self.mod.is_show_only = True
        return self

    def turn_off_show_only(self):
        self.mod.is_show_only = False
        return self

    def turn_on_debug(self):
        self.mod.is_debug = True
        return self

    def turn_off_debug(self):
        self.mod.is_debug = False
        return self

    def turn_on_transparent(self):
        self.mod.is_transparent = True
        return self

    def turn_off_transparent(self):
        self.mod.is_transparent = False
        return self

    # Shorthand
    def disable(self):
        return self.turn_on_disable()

    def show_only(self):
        return self.turn_on_show_only()

    def debug(self):
        return self.turn_on_debug()

    def transparent(self):
        return self.turn_on_transparent()



# -*- coding: utf-8 -*-
import openpyscad.base as base

__all__ = ['Translate', 'Rotate', 'Scale', 'Resize', 'Mirror', 'Color', 'Offset', 'Hull', 'Minkowski', 'Linear_Extrude', 'Rotate_Extrude']


class _Transformation(base.BaseObject):
    pass


Transformation = _Transformation


# Transformations
class Translate(_Transformation):
    pass


class Rotate(_Transformation):
    pass


class Scale(_Transformation):
    pass


class Resize(_Transformation):
    pass


class Mirror(_Transformation):
    pass


class Color(_Transformation):
    pass


class Offset(_Transformation):

    def _validate_append(self, obj):
        from .shapes_2d import Shape2dObject
        if not isinstance(obj, Shape2dObject):
            raise TypeError('Appended object must be a instance of Shape2dObject.')


class Hull(_Transformation):
    pass


class Minkowski(_Transformation):
    pass


class Linear_Extrude(_Transformation):
    def _validate_append(self, obj):
        from .shapes_2d import Shape2dObject
        if not isinstance(obj, (Shape2dObject, Transformation)):
            raise TypeError('Appended object must be a instance of Shape2dObject or Transformation.')


class Rotate_Extrude(_Transformation):
    def _validate_append(self, obj):
        from .shapes_2d import Shape2dObject
        if not isinstance(obj, (Shape2dObject, Transformation)):
            raise TypeError('Appended object must be a instance of Shape2dObject.')



from __future__ import absolute_import

from .base import _BaseObject

__all__ = ['Circle', 'Square', 'Polygon', 'Text']


class _Shape2dObject(_BaseObject):
    pass


Shape2dObject = _Shape2dObject


class Circle(_Shape2dObject):
    pass


class Square(_Shape2dObject):
    pass


class Polygon(_Shape2dObject):
    pass


class Text(_Shape2dObject):
    pass



from __future__ import absolute_import

from .base import _BaseObject

__all__ = ['Sphere', 'Cube', 'Cylinder', 'Polyhedron', 'Surface']


# 3D
class _Shape3dObject(_BaseObject):
    pass


class Sphere(_Shape3dObject):
    pass


class Cube(_Shape3dObject):
    pass


class Cylinder(_Shape3dObject):
    pass


class Polyhedron(_Shape3dObject):
    pass


class Surface(_Shape3dObject):
    pass



from math import cos, sin, pi

import openpyscad as ops


class Custom2dShapes(object):

    @staticmethod
    def regular_polygon(num, r):
        points = list()
        for i in range(num):
            a_deg = i * 360 / num
            a_rad = a_deg * 2 * pi / 360
            points += [[r * cos(a_rad), r * sin(a_rad)]]
        regular_poly = ops.Polygon(points)
        return(regular_poly)

    @staticmethod
    def star(num, radii):
        points = list()
        for i in range(num):
            a_deg = i * 360 / num
            a_rad = a_deg * 2 * pi / 360
            r = radii[i % len(radii)]
            points += [[r * cos(a_rad), r * sin(a_rad)]]
            star_object = ops.Polygon(points)
        return(star_object)



import openpyscad as ops


class Custom3dShapes(object):

    @staticmethod
    def dice(edge=15, fn=32):
        """ dice
        """
        edge = float(edge)
        # dice
        c = ops.Cube(edge, center=True)
        s = ops.Sphere(edge * 3 / 4, center=True)
        dice = c & s
        # points
        c = ops.Circle(edge / 12, _fn=fn)
        h = 0.7
        point = c.linear_extrude(height=h)
        point1 = point.translate([0, 0, edge / 2 - h / 2])
        point2_1 = point1.rotate(a=90, v=[1, 0, 0]).translate([edge / 6, 0, edge / 6])
        point2_2 = point2_1.mirror([-edge / 6, 0, -edge / 6])
        point2 = point2_1 + point2_2
        point3 = point2.rotate(a=90, v=[0, 0, 1]) + point1.rotate(a=90, v=[0, 1, 0])
        point4_12 = point2.rotate(a=-90, v=[0, 0, 1])
        point4 = point4_12 + point4_12.mirror([0, 1, 0])
        point5_123 = point3.rotate(a=90, v=[0, 0, 1])
        point5 = point5_123 + point5_123.mirror([1, 0, 0])
        point6_1 = point.translate([0, 0, -(edge / 2 + h / 2)]).translate([0, edge / 6, 0])
        point6_2 = point6_1.translate([edge / 4, 0, 0])
        point6_3 = point6_1.translate([-edge / 4, 0, 0])
        point6_123 = point6_1 + point6_2 + point6_3
        point6_456 = point6_123.mirror([0, 1, 0])
        point6 = point6_123 + point6_456
        dice_with_holes = dice - point1 - point2 - point3 - point4 - point5 - point6
        dice_with_holes = dice_with_holes.mirror([0, 0, 1])
        return(dice_with_holes)



# -*- coding: utf-8 -*-
from __future__ import absolute_import

# Python 2 and 3:
from six import with_metaclass
import os


from .modifiers import ModifierMixin

__all__ = ['Empty', 'BaseObject', 'Scad', 'Import']
INDENT = '    '


class MetaObject(type):

    object_definition = {
        # Bool
        'union': ('union', (), True),
        'difference': ('difference', (), True),
        'intersection': ('intersection', (), True),
        # Transforms
        'translate': ('translate', ('v', ), True),
        'rotate': ('rotate', ('a', 'v'), True),
        'scale': ('scale', ('v', ), True),
        'resize': ('resize', ('newsize', 'auto'), True),
        'mirror': ('mirror', ('__axis', ), True),
        'color': ('color', ('__color', 'a'), True),
        'offset': ('offset', ('r', 'chamfer', '_fn'), True),
        'minkowski': ('minkowski', (), True),
        'hull': ('hull', (), True),
        'linear_extrude': ('linear_extrude', ('height', 'center',
                           'convexity', 'twist', 'slices', 'scale'),
                           True),
        'rotate_extrude': ('rotate_extrude', ('angle', 'convexity',
                           '_fn'), True),
        # 2D
        'circle': ('circle', ('r', 'd', '_fn'), False),
        'square': ('square', ('size', 'center'), False),
        'polygon': ('polygon', ('points', 'paths', 'convexity'), False),
        'text': ('text',
                 ('text', 'size', 'font', 'halign', 'valign', 'spacing',
                  'direction', 'language', 'script', '_fn'),
                 False),
        # 3D
        'sphere': ('sphere', ('r', 'd', 'center', '_fa', '_fs', '_fn'), False),
        'cube': ('cube', ('size', 'center'), False),
        'cylinder': ('cylinder',
                     ('h', 'r', 'r1', 'r2', 'd', 'd1', 'd2',
                      'center', '_fa', '_fs', '_fn'),
                     False
                     ),
        'scad': ('scad', ('scadfile', 'version'), False),
        'import': ('import', ('file', 'convexity'), False),
        'surface': ('surface', ('file', 'center', 'invert', 'convexity'), False),
        'polyhedron': ('polyhedron',
                       ('points', 'triangles', 'faces', 'convexity'),
                       False)
    }

    def __new__(mcs, name, bases, attr):
        if name[0] != '_':
            definition = MetaObject.object_definition[name.lower()]
            attr['_name'] = definition[0]
            attr['_properties'] = definition[1]
            for param in definition[1]:
                attr[param] = None
            attr['has_child'] = definition[2]
        return type.__new__(mcs, name, bases, attr)


class _BaseObject(with_metaclass(MetaObject, ModifierMixin, object)):

    def __init__(self, *args, **kwargs):
        super(_BaseObject, self).__init__()
        self.modules = list()
        for k, v in kwargs.items():
            if hasattr(self.__class__, k):
                setattr(self, k, v)

        if len(args) > 0:
            for i, k in enumerate(args):
                setattr(self, self._properties[i], args[i])

        self.children = []

    def _retrieve_value(self, name):
        val = getattr(self, name)
        if val is None:
            return None
        try:
            if isinstance(val, str):
                return '"{}"'.format(val)
        except Exception as e:
            print(e)
            if isinstance(val, str):
                return '"{}"'.format(val)

        return '{}'.format(val)

    def _get_params(self, fp=None):
        valid_keys = list(filter(lambda x: getattr(self, x) is not None, self._properties))

        def is_no_keyword_args(arg_name):
            if arg_name[0] == '_' and arg_name[1] == '_':
                return True
            return False

        def is_keyword_args(arg_name):
            return not is_no_keyword_args(arg_name)

        def convert_special_args(arg_name):
            if arg_name[0] == '_':
                if arg_name[1] != '_':
                    return '$' + arg_name[1:]
            return arg_name

        def _get_attr(self, x, fp):
            if x == 'scadfile':
                scadfile = getattr(self, x)

                def rename_scadfile(scadfile):
                    sf = ''.join(os.path.basename(scadfile).split('.')[:-1])
                    scadfile_renamed = sf.lower().strip('_').strip('-')
                    return(scadfile_renamed)
                with open(scadfile) as f:
                    content = f.readlines()
                    content = ''.join(content).rstrip('\n')
                    sc = rename_scadfile(scadfile)
                    module = 'module {sc}() {{{content};}}\n'.format(**{'content': content, 'sc': sc})
                    module = module.replace(';;', ';')
                    self.modules.append(module)
                    if fp is not None:
                        fp.write(module)
                    content = '{}()'.format(sc)
                    return(content)
            elif x == 'file':
                content = getattr(self, x)
                if not content.startswith('"'):
                    content = '"' + content
                if not content.endswith('"'):
                    content = content + '"'
                return(content)
            else:
                return(getattr(self, x))

        args = ''
        # no-keyword args
        no_kw_args = list(filter(lambda x: is_no_keyword_args(x), valid_keys))
        args += ' '.join(map(lambda x: '{},'.format(self._retrieve_value(x)), no_kw_args))[:-1]

        # keyword args
        kw_args = filter(lambda x: is_keyword_args(x), valid_keys)
        args += ' '.join(map(lambda x: '{}={},'.format(convert_special_args(x), _get_attr(self, x, fp)), kw_args))[:-1]
        args = args.replace('scadfile=', '')
        return args

    def _get_children_content(self, indent_level=0, fp=None):
        _content = ''
        if len(self.children) > 0:
            for child in self.children:
                _content += child.dumps(indent_level, fp)

        return _content

    def _get_content(self, indent_level=0, fp=None):
        if len(self.children) == 0:
            return ''
        else:
            return '{{\n{children}{indent}}}'.format(
                children=self._get_children_content(indent_level + 1, fp=fp),
                indent=INDENT * indent_level
            )

    def _validate_append(self, obj):
        """Override if any validation in append operation is required.
        :param obj:
        """

    def append(self, obj):
        if not self.has_child:
            raise TypeError('This object can not have any children.')
        else:
            self._validate_append(obj)
            if isinstance(obj, (list, tuple, set)):
                for o in obj:
                    self.append(o)
            else:
                self.children.append(obj)
            return self

    def dump(self, fp):
        dumps = self.dumps(fp=fp)
        fp.write(dumps)

    def dumps(self, indent_level=0, fp=None):
        if self._name == 'scad':
            return '{indent}{prefix}{params};\n'.format(
                indent=INDENT * indent_level,
                prefix=self.mod.get_prefix(),
                params=self._get_params(fp).replace('True', 'true')
            )
        else:
            return '{indent}{prefix}{op_name}({params}){content};\n'.format(
                indent=INDENT * indent_level,
                prefix=self.mod.get_prefix(),
                op_name=self._name,
                params=self._get_params(fp).replace('True', 'true'),
                content=self._get_content(indent_level, fp=fp)
            )

    def write(self, filename, with_print=False):
        with open(filename, 'w') as fp:
            self.dump(fp)
        if with_print:
            print(self.dumps())

    def clone(self):
        import copy
        return copy.deepcopy(self)

    def equals(self, other):
        for prop in self._properties:
            if self.__class__.__name__ != other.__class__.__name__:
                return False

            if getattr(self, prop) != getattr(other, prop):
                return False
        return True

    def __str__(self):
        return self.dumps()

    def __add__(self, other):
        from .boolean import Union
        if isinstance(self, _Empty):
            return other.clone()

        elif isinstance(self, Union):
            cloned = self.clone()
            cloned.append(other)
            return cloned
        else:
            return Union().append(self).append(other)

    def __sub__(self, other):
        from .boolean import Difference
        if isinstance(self, _Empty):
            return other.clone()

        elif isinstance(self, Difference):
            cloned = self.clone()
            cloned.append(other)
            return cloned
        else:
            return Difference().append(self).append(other)

    def __and__(self, other):
        from .boolean import Intersection
        if isinstance(self, _Empty):
            return other.clone()

        elif isinstance(self, Intersection):
            cloned = self.clone()
            cloned.append(other)
            return cloned
        else:
            return Intersection().append(self).append(other)

    def translate(self, *args, **kwargs):
        from .transformations import Translate
        return Translate(*args, **kwargs).append(self)

    def rotate(self, *args, **kwargs):
        from .transformations import Rotate
        return Rotate(*args, **kwargs).append(self)

    def scale(self, *args, **kwargs):
        from .transformations import Scale
        return Scale(*args, **kwargs).append(self)

    def resize(self, *args, **kwargs):
        from .transformations import Resize
        return Resize(*args, **kwargs).append(self)

    def mirror(self, *args, **kwargs):
        from .transformations import Mirror
        return Mirror(*args, **kwargs).append(self)

    def color(self, *args, **kwargs):
        from .transformations import Color
        return Color(*args, **kwargs).append(self)

    def offset(self, *args, **kwargs):
        from .transformations import Offset
        return Offset(*args, **kwargs).append(self)

    def minkowski(self, *args, **kwargs):
        from .transformations import Minkowski
        return Minkowski(*args, **kwargs).append(self)

    def hull(self, *args, **kwargs):
        from .transformations import Hull
        return Hull(*args, **kwargs).append(self)

    def linear_extrude(self, *args, **kwargs):
        from .transformations import Linear_Extrude
        return Linear_Extrude(*args, **kwargs).append(self)

    def rotate_extrude(self, *args, **kwargs):
        from .transformations import Rotate_Extrude
        return Rotate_Extrude(*args, **kwargs).append(self)


BaseObject = _BaseObject


class _Empty(_BaseObject):
    pass


Empty = _Empty


class Scad(_BaseObject):
    pass


class Import(_BaseObject):
    pass



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import openpyscad.base as base

__all__ = ['Union', 'Difference', 'Intersection']


class Union(base.BaseObject):
    pass


class Difference(base.BaseObject):
    pass


class Intersection(base.BaseObject):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['openpyscad']

package_data = \
{'': ['*']}

install_requires = \
['six>=1.14.0,<2.0.0']

setup_kwargs = {
    'name': 'openpyscad',
    'version': '0.4.0',
    'description': 'Python library to generate OpenSCAD source code',
    'long_description': '[![Build Status](https://travis-ci.org/taxpon/openpyscad.svg?branch=develop)](https://travis-ci.org/taxpon/openpyscad) [![Coverage Status](https://coveralls.io/repos/github/taxpon/openpyscad/badge.svg?branch=develop)](https://coveralls.io/github/taxpon/openpyscad?branch=develop) [![Python2](https://img.shields.io/badge/python-2-blue.svg)](#) [![Python3](https://img.shields.io/badge/python-3-blue.svg)](#)\n# OpenPySCAD\nPython library to generate [OpenSCAD](http://www.openscad.org/) source code. This library provides intuitive interface when you handle 3D data.\nOpenPySCAD supports python3(3.5+).\n\n## Install\n```bash\npip install openpyscad\n```\n\n## How to use\n- Write python code as follows:\n```python\nimport openpyscad as ops\nc1 = ops.Cube([10, 20, 10])\nc2 = ops.Cube([20, 10, 10])\n(c1 + c2).write("sample.scad")\n```\n\n- Generated code will be written in the "sample.scad". OpenSCAD can detect the change of your code and reload automatically. That\'s so cool :D\n```openscad\nunion(){\n    cube([10, 20, 10]);\n    cube([20, 10, 10]);\n};\n```\n\n## Generated code examples\n\n### 3D Shapes\n\nPython:\n```python\nSphere(r=10, _fn=100)\nCube([10, 10, 10])\nCylinder(h=10, r=10)\np = Polyhedron(\n    points=[\n        [10, 10, 0], [10, -10, 0], [-10, -10, 0], [-10, 10, 0],  [0, 0, 10]\n    ],\n    faces=[\n        [0, 1, 4], [1, 2, 4], [2, 3, 4], [3, 0, 4],  [1, 0, 3], [2, 1, 3]\n    ]\n)\n```\n\nGenerated OpenSCAD code:\n```openscad\nsphere(r=10, $fn=100);\ncube(size=[10, 10, 10]);\ncylinder(h=10, r=10);\npolyhedron(points=[[10, 10, 0], [10, -10, 0], [-10, -10, 0], [-10, 10, 0], [0, 0, 10]], faces=[[0, 1, 4], [1, 2, 4], [2, 3, 4], [3, 0, 4], [1, 0, 3], [2, 1, 3]]);\n```\n\n### Boolean Operations\n\nPython:\n```python\n# Union\nCube([20, 10, 10]) + Cube([10, 20, 10])\n\n# You can also write like this\nu = Union()\nu.append(Cube[20, 10, 10])\nu.append(Cube[10, 20, 10])\n\n# Difference\nCube([20, 10, 10]) - Cube([10, 20, 10])\n\n# You can also write like this\ni = Difference()\ni.append(Cube[20, 10, 10])\ni.append(Cube[10, 20, 10])\n\n# Intersection\nCube([20, 10, 10]) & Cube([10, 20, 10])\n\n# You can also write like this\ni = Intersection()\ni.append(Cube[20, 10, 10])\ni.append(Cube[10, 20, 10])\n```\n\nGenerated OpenSCAD code:\n```openscad\n// Union\nunion(){\n    cube([20, 10, 10])\n    cube([10, 20, 10])\n};\n\n// Difference\ndifference(){\n    cube([20, 10, 10]);\n    cube([10, 20, 10]);\n};\n\n// Intersection\nintersection(){\n    cube([20, 10, 10]);\n    cube([10, 20, 10]);\n};\n```\n\n### Transformations\n\nPython:\n```python\n# Translate\nCube([20, 10, 10]).translate([10, 10, 10])\n\n# Rotate\nCube([20, 10, 10]).rotate([0, 0, 45])\n\n# Scale\nCube([20, 10, 10]).scale([2, 1, 1])\n\n# Resize\nCube([20, 10, 10]).resize([2, 1, 1])\n\n# Mirror\nCube([20, 10, 10]).mirror([1, 1, 1])\n\n# Color\nCube([20, 10, 10]).color("Red")\n\n# Offset\nCircle(10).offset(10)\n```\n\nGenerated OpenSCAD code:\n```openscad\n// Translate\ntranslate(v=[10, 10, 10]){\n    cube([20, 10, 10]);\n};\n\n// Rotate\nrotate(v=[0, 0, 45]){\n    cube([20, 10, 10]);\n};\n\n// Scale\nscale(v=[2, 1, 1]){\n    cube([20, 10, 10]);\n};\n\n// Resize\nresize(newsize=[2, 1, 1]){\n    cube(size=[20, 10, 10]);\n};\n\n// Mirror\nmirror([1, 1, 1]){\n    cube(size=[20, 10, 10]);\n};\n\n// Color\ncolor("Red"){\n    cube(size=[20, 10, 10]);\n};\n\n// Offset\noffset(r=10){\n    circle(r=10);\n};\n```\n\n### Modifiers\nOpenPySCAD provides [modifiers](https://en.wikibooks.org/wiki/OpenSCAD_User_Manual/Modifier_Characters) interfaces ("*", "!", "#" and "%").\n\nPython:\n```python\nc1 = Cube(10)\nc1.disable()         # add "*" character\nc1.show_only()       # add "!" character\nc1.is_debug()        # add "#" character\nc1.is_transparent()  # add "&" character\n```\n\n## Interested in contribution?\nPlease read [CONTRIBUTING.md](./CONTRIBUTING.md). \n\n',
    'author': 'Takuro Wada',
    'author_email': 'taxpon@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/taxpon/openpyscad',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
